from __future__ import annotations

import numpy as np


def ste(x: np.ndarray, a: float, f_infty: float, tau: float, beta: float) -> np.ndarray:
    return a*((1-f_infty) * np.exp(-(x/tau)**beta) + f_infty)


def pulse_attn(freq: np.ndarray, t_pulse: float):
    # cf. Schmitt-Rohr/Spieß eq. 2.126; omega_1 * t_p = pi/2
    pi_half_squared = np.pi**2 / 4
    omega = 2 * np.pi * freq

    numerator = np.sin(np.sqrt(pi_half_squared + omega**2 * t_pulse**2 / 2))
    denominator = np.sqrt(pi_half_squared + omega**2 * t_pulse**2 / 4)

    return np.pi * numerator / denominator / 2
